using System;
using System.IO;
using System.Drawing; // pour Bitmap
using System.Drawing.Imaging; // pour ImageFormat
using System.Resources;
using System.Reflection;

using Gtk;
using GtkSharp;

class PixbufWidget : Gtk.DrawingArea
{
	private Gdk.Pixbuf pb, scaledpb;
	int w = 0, h = 0;

	public PixbufWidget(Gdk.Pixbuf _pb, int _x, int _y, int _w, int _h)
	{
		pb = new Gdk.Pixbuf(_pb, _x, _y, _w, _h);
		Size(pb.Width, pb.Height);
	}
 
	protected override bool OnExposeEvent(Gdk.EventExpose args)
	{
		Gdk.Window win = args.Window;

		if(w != Allocation.Width || h != Allocation.Height)
		{
			w = Allocation.Width;
			h = Allocation.Height;
 
			scaledpb = pb.ScaleSimple(w, h, Gdk.InterpType.Hyper);
		}

		win.DrawPixbuf(Style.BaseGC(StateType.Normal), scaledpb, 0, 0, 0, 0, w, h, Gdk.RgbDither.Normal, 0, 0);
 
		return true;
	} 
}

class RoundWindow : Gtk.Window
{
	Gdk.Pixmap circle;
	int radius;
	int w = 0, h = 0;

	public RoundWindow (string _name, int _radius)
	 : base(Gtk.WindowType.Toplevel)
	{
		Name = _name;
		radius = _radius;

		// Quick and dirty bitmap for fancy edges
		circle = new Gdk.Pixmap(null, 2 * radius + 1, 2 * radius + 1, 1);
		using(Gdk.GC gc = new Gdk.GC(circle))
		{
			Gdk.Colormap colormap = Gdk.Colormap.System;
			Gdk.Color white = new Gdk.Color(255, 255, 255);
			colormap.AllocColor(ref white, true, true);

			Gdk.Color black = new Gdk.Color(0, 0, 0);
			colormap.AllocColor(ref black, true, true);

			gc.Foreground = black;
			circle.DrawRectangle(gc, true, 0, 0, 2 * radius + 1, 2 * radius + 1);

			gc.Foreground = white;
			for(int j = 0; j < 2 * radius + 1; j++)
				for(int i = 0; i < 2 * radius + 1; i++)
				{
					if((i - radius) * (i - radius) + (j - radius) * (j - radius) > radius * radius)
						continue;
					circle.DrawPoint(gc, i, j);
				}
		}
	}

	// Shape window (put round corners)
	protected override void OnSizeAllocated(Gdk.Rectangle alloc)
	{
		base.OnSizeAllocated(alloc);

		if(w == alloc.Width && h == alloc.Height)
			return;

		w = alloc.Width;
		h = alloc.Height;

		Gdk.Pixmap bm = new Gdk.Pixmap(GdkWindow, w, h, 1);
		using(Gdk.GC gc = new Gdk.GC(bm))
		{
			Gdk.Colormap colormap = Gdk.Colormap.System;

			// Create and allocate the colors
			Gdk.Color white = new Gdk.Color(255, 255, 255);
			colormap.AllocColor(ref white, true, true);

			Gdk.Color black = new Gdk.Color(0, 0, 0);
			colormap.AllocColor(ref black, true, true);

			// Fill background
			gc.Foreground = white;
			bm.DrawRectangle(gc, true, 0, 0, w, h);

			// Draw circles corners
			bm.DrawDrawable(gc, circle, 0, 0, 0, 0, radius, radius);
			bm.DrawDrawable(gc, circle, radius + 1, 0, w - radius, 0, radius, radius);
			bm.DrawDrawable(gc, circle, 0, radius + 1, 0, h - radius, radius, radius);
			bm.DrawDrawable(gc, circle, radius + 1, radius + 1, w - radius, h - radius, radius, radius);
		}
		ShapeCombineMask(bm, 0, 0);
		bm.Dispose();
	}
}

public class Foo
{
	RoundWindow window;
	Gdk.Cursor move, resize;

	public static void Main()
	{
		new Foo();
	}

	public Foo()
	{
		Application.Init();

		window = new RoundWindow("Foo 3000", 19);
		window.Decorated = false; // no window border
		window.DeleteEvent += QuitFoo;

		// Put everything in an EventBox to get mouse events
		EventBox bg = new EventBox();
		bg.ButtonPressEvent += ClickFoo;
		bg.AddEvents((int)Gdk.EventMask.PointerMotionMask); // needed
		bg.MotionNotifyEvent += MouseFoo;
		window.Add(bg);

		// Create a 3x3 table with pixmaps
		Table table = new Table(3, 3, false);
		Gdk.Pixbuf pixbuf = LoadPixbuf("image.png");
		int[] xcut = { 0, 32, 302, pixbuf.Width };
		int[] ycut = { 0, 148, 507, pixbuf.Height };
		for(uint j = 0; j < 3; j++)
			for(uint i = 0; i < 3; i++)
			{
				if(i == 1 && j == 1)
					continue;
				AttachOptions ox = AttachOptions.Fill,
					      oy = AttachOptions.Fill;
				// Only resize edge cells
				if(i == 1)
					ox |= AttachOptions.Expand | AttachOptions.Shrink;
				if(j == 1)
					oy |= AttachOptions.Expand | AttachOptions.Shrink;
				PixbufWidget tmp = new PixbufWidget(pixbuf, xcut[i], ycut[j], xcut[i + 1] - xcut[i], ycut[j + 1] - ycut[j]);
				table.Attach(tmp, i, i + 1, j, j + 1, ox, oy, 0, 0);
			}
		bg.Add(table);

		VBox vbox = new VBox(false, 2);
		Frame frame = new Frame("Test");
		Label label = new Label("This is a test frame.\nIt will eventually\ncontain a lot of\nGTK widgets, which\nis pretty awesome.");
		//label.Justify = Justification.Left; // XXX: doesn't seem to work with GTK#
		label.Wrap = true;
		frame.Add(label);
		vbox.Add(frame);
		Button button = new Button("Hit me to quit");
		button.Clicked += new System.EventHandler(QuitFoo);
		vbox.Add(button);
		table.Attach(vbox, 1, 2, 1, 2);

		window.ShowAll();

		move = new Gdk.Cursor(Gdk.CursorType.Hand1);
		resize = new Gdk.Cursor(Gdk.CursorType.BottomRightCorner);

		Application.Run();
	}

	private void QuitFoo(object o, System.EventArgs args)
	{
		Application.Quit();
	}

	[GLib.ConnectBefore]
	private void MouseFoo(object o, System.EventArgs _args)
	{
		// Change the mouse pointer appearance based on totally
		// arbitrary values
		MotionNotifyEventArgs args = (MotionNotifyEventArgs)_args;

		if(args.Event.Y < 24)
		{
			window.GdkWindow.Cursor = move;
			return;
		}

		if(args.Event.X + 24 > window.Allocation.Width
		    && args.Event.Y + 24 > window.Allocation.Height)
		{
			window.GdkWindow.Cursor = resize;
			return;
		}

		window.GdkWindow.Cursor = null;
	}

	[GLib.ConnectBefore]
	private void ClickFoo(object o, System.EventArgs _args)
	{
		ButtonPressEventArgs args = (ButtonPressEventArgs)_args;
		if(args.Event.Type != Gdk.EventType.ButtonPress)
			return;
		if(args.Event.Button != 1)
			return;

		// Dragging or resizing window
		if(args.Event.Y < 24)
		{
			window.BeginMoveDrag(1, (int)args.Event.XRoot, (int)args.Event.YRoot, args.Event.Time);
			return;
		}

		if(args.Event.X + 24 > window.Allocation.Width
		    && args.Event.Y + 24 > window.Allocation.Height)
		{
			window.BeginResizeDrag(Gdk.WindowEdge.SouthEast, (int)args.Event.Button, (int)args.Event.XRoot, (int)args.Event.YRoot, args.Event.Time);
			return;
		}
	}

	// Load an image into a GdkPixbuf
	private static Gdk.Pixbuf LoadPixbuf(string imagename)
	{
		using (MemoryStream stream = new MemoryStream())
		{
			Assembly a = Assembly.GetExecutingAssembly();
			Stream str = a.GetManifestResourceStream(imagename);
			Bitmap tmp = new Bitmap(str);
			tmp.Save(stream, ImageFormat.Bmp);
			stream.Position = 0;
			return new Gdk.Pixbuf(stream);
		}
	}
}
